# This code creates AA data for estimation

# Remove everything
# rm(list = ls())

library(RPostgreSQL)
library(dplyr)
library(tidyr)
library(psych)

pg <- dbConnect(PostgreSQL())

# Define PG data ----

# Project schema
rs <- dbGetQuery(pg, "SET search_path TO non_gaap, public")
est_merged <- tbl(pg, "est_merged")
detail_set <- tbl(pg, "detail_set")

# Define params ----

q_bea_at_cutoff <- 0.10 # cutoff for intang. capital intensity

# Compute non-GAAP adjustments ----

aa_type <-     
    detail_set %>%
    # Combine all reconciliation amounts per non-GAAP metric
    group_by(gvkey, datadate, metric_key, metric_type) %>% 
    summarise(psi_aa = sum(value, na.rm = TRUE)/1e6) %>% 
    ungroup()

# Select which non-GAAP amount to use in estimation ----

aa_type_post_bcgw <- aa_type %>% 
    # Filter data to be post BCGW sample
    filter(datadate > as.Date('2016-05-31')) %>%     
    # Rank AA metrics based on proximity to BCGW definition for post-BCGW coverage    
    mutate(metric_rank = ifelse(
        metric_type == "EPS", 1, ifelse(
        metric_type == "Other EPS", 2, ifelse(
        metric_type == "Net Income", 3, ifelse(
        metric_type == "Operating Income", 4, ifelse(
        metric_type == "Income Other Than Net Income, Operating Income Or Segment Income", 5, ifelse(
        metric_type == "EBIT", 6, ifelse(
        metric_type == "Adjusted EBITA", 7, ifelse(
        metric_type == "Adjusted EBITA", 8, ifelse(
        metric_type == "EBITDA", 9, 
        99)))))))))) %>%  # Set all other metrics to '99' to exclude it from analysis
    # Keep smallest metric_type per firm-year
    group_by(gvkey, datadate) %>% 
    filter(metric_rank == min(metric_rank)) %>% 
    ungroup() %>% 
    # Keep smallest adjustment in case of ties
    group_by(gvkey, datadate, metric_rank) %>% 
    filter(psi_aa == min(psi_aa)) %>% 
    ungroup() %>%
    # Keep smallest metric_key for ties
    # NOTE: this is arbitrary but the information is the same in both NG amounts
    group_by(gvkey, datadate, metric_rank, psi_aa) %>% 
    filter(metric_key == min(metric_key)) %>% 
    ungroup() %>% 
    select(-metric_rank)

# Select AA adjustment that is closest to BCGW if within timeframe
aa_type_pre_bcgw <- aa_type %>% 
    # Merge in psi from BCGW
    left_join(est_merged %>% select(gvkey, datadate, psi)) %>% 
    # Compute absolute distance between NG data
    mutate(d = abs(psi_aa - psi)) %>%
    # Keep smallest distance
    group_by(gvkey, datadate) %>% 
    filter(d == min(d)) %>% 
    select(-d, -psi) %>%     
    # Keep smallest metric_key for ties
    # NOTE: this is arbitrary but the information is the same in both NG amounts
    group_by(gvkey, datadate, psi_aa) %>% 
    filter(metric_key == min(metric_key)) %>% 
    ungroup()

# Combine pre- and post-bcgw data
aa_type_alt <- union(aa_type_pre_bcgw, aa_type_post_bcgw)

# Calculate cash and non-cash adjustments ----

aa_cash <- 
    detail_set %>% 
    # Sum adjustments by cash/non-cassh
    group_by(gvkey, datadate, metric_key, metric_type, cash) %>% 
    summarise(psi_aa_sub = sum(value)/1e6) %>% 
    arrange(gvkey, datadate, metric_key, metric_type, cash)

# Merge cash/non-cash split
aa_type_alt <- 
    aa_type_alt %>% 
    # Merge in cash adjustments
    left_join(aa_cash %>%
                  filter(cash ==1)) %>%
    rename(psi_aa_cash = psi_aa_sub) %>% 
    select(-cash) %>% 
    # Merge in non-cash adjustments
    left_join(aa_cash %>%
              filter(cash ==0)) %>%
    rename(psi_aa_noncash = psi_aa_sub) %>% 
    select(-cash) %>% 
    # Set missing adjustments to 0
    mutate(psi_aa_cash = ifelse(is.na(psi_aa_cash), 0, psi_aa_cash),
       psi_aa_noncash = ifelse(is.na(psi_aa_noncash), 0, psi_aa_noncash))

# Merge AA data to est_merged and compute variable ----
# NOTE: This code is drawn from create_firms.R

est_merged_alt <- 
    est_merged %>% 
    # Merge in cash adjustments
    left_join(aa_type_alt %>%
                  select(-metric_key, -metric_type)) %>%
    # Determine start and end dates of AA data
    left_join(aa_type_alt %>% 
                      group_by(gvkey) %>% 
                      summarise(aa_start = min(datadate),
                                aa_end = max(datadate)) %>% 
                      ungroup()) %>% 
    # Indicate if firm provides non-GAAP
    mutate(mgr_exclude_aa = ifelse(is.na(psi_aa), 0, 1)) %>% 
    # Set missing {psi_aa, psi_aa_cash, psi_aa_noncash} to 0
    mutate(psi_aa = ifelse(is.na(psi_aa), 0, psi_aa),
           psi_aa_cash = ifelse(is.na(psi_aa_cash), 0, psi_aa_cash),
           psi_aa_noncash = ifelse(is.na(psi_aa_noncash), 0, psi_aa_noncash)) %>% 
    # Compute non-GAAP earnings (piing)
    mutate(piing_aa = pii + psi_aa,
           piing_aa_cash = pii + psi_aa_cash,
           piing_aa_noncash = pii + psi_aa_noncash) %>%  
    # Scale {psi_aa, psi_aa_cash, psi_aa_noncash} by capital
    mutate_at(.vars = vars(psi_aa, psi_aa_cash, psi_aa_noncash, 
                           piing_aa, piing_aa_cash, piing_aa_noncash), 
              .funs = funs(q_bea = if_else(. != 0, ./NULLIF(q_bea, 0), 0), 
                           q_epw = if_else(. != 0, ./NULLIF(q_epw, 0), 0), 
                           q_ep = if_else(. != 0, ./NULLIF(q_ep, 0), 0), 
                           q_noinit = if_else(. != 0, ./NULLIF(q_noinit, 0), 0))) %>% 
    group_by(gvkey, fqtr) %>%
    arrange(datadate) %>%    
    # Compute iteract. terms for pricing moments
    mutate(psi_aa2 = psi_aa^2,
           psi_aa_cash2 = psi_aa_cash^2,
           psi_aa_noncash2 = psi_aa_noncash^2,
           piipsi_aa = pii*psi_aa,
           piipsi_aa_cash = pii*psi_aa_cash,
           piipsi_aa_noncash = pii*psi_aa_noncash,
           piing_aa2 = piing_aa^2, 
           piing_aa_cash2 = piing_aa_cash^2, 
           piing_aa_noncash2 = piing_aa_noncash^2, 
           piipiing_aa = pii*piing_aa,
           piipiing_aa_cash = pii*piing_aa_cash,
           piipiing_aa_noncash = pii*piing_aa_noncash) %>% 
    # Compute growth rates for {w, d, pii, psi, piing} for firm-quarters
    # Compute growth rates as in Stephen Terry's JMP
    # WARNING: cannot send . and ._lag to sql
    # Growth based on raw variables
    mutate_at(.vars = vars(psi_aa, psi_aa_cash, psi_aa_noncash,
                           psi_aa2, psi_aa_cash2, psi_aa_noncash2,
                           piipsi_aa, piipsi_aa_cash, piipsi_aa_noncash,
                           piing_aa, piing_aa_cash, piing_aa_noncash,
                           piing_aa2, piing_aa_cash2, piing_aa_noncash2,
                           piipiing_aa, piipiing_aa_cash, piipiing_aa_noncash),
          .funs = funs(gr1 = if_else(. == 0 & sql(lag(.)) == 0, 0, 
                                     2*(. - sql(lag(.)))/
                                         (sql(NULLIF(abs(1.0*.) + lag(abs(1.0*.)), 0)))))) %>% 
    # Compute changes in {pii_q, psi_q, d_q, piing_q} for X-differences for rho_X for firm-quarters
    mutate_at(.vars = vars(matches("^psi_aa_q|^psi_aa_cash_q|^psi_aa_noncash_q|
                                   ^piing_aa_q|^piing_aa_cash_q|^piing_aa_noncash_q")), 
              .funs = funs(lag03 = . - lag(., 3), 
                           lag12 = lag(., 1) - lag(., 2))) %>%
        ungroup() %>%
    compute()


# Filter data ----
# NOTE: This code is largely copied from est_set.R

dbGetQuery(pg, "DROP TABLE IF EXISTS aa_set")

est_merged_alt <-
    est_merged_alt %>% 
    # Keep only those observations between start and end dates
    filter(aa_start <= datadate & datadate <= aa_end) %>%
    select(-aa_start, -aa_end, -bcgw_start, -bcgw_end) %>% 
    # Exclude regulated utilities and non-operating establ. as in Peters and Taylor (2017)
    # WARNING: Eisfeldt and Papanikolaou (2014) keep finance. Peters and Taylor (2017) drop finance
    filter(!is.na(ind_ep6) & ind_ep6 != "Utilities" & ind_ep6 != "Finance" & ind_ep6 != "Non-operating est.") %>%
    # Exclude missing at, missing sales, at < 5 mil as in Peters and Taylor (2017)
    filter(!is.na(at) & !is.na(sale) & at > 5) %>%
    # Keep only above intangible intensity cutoff by BEA criterion
    group_by(gvkey) %>%
    mutate(q_bea_at_min = min(q_bea_at)) %>%
    ungroup() %>%
    filter(q_bea_at_min >= q_bea_at_cutoff) %>%
    select(-q_bea_at_min) %>%
    collect() %>%
    # Winsorize at 1-/99-
    mutate_at(.vars = vars(-gvkey, -datadate, -fyearq, -fqtr, -sich, -ind_ep6,
                           -consens_n, -company_id, -executive_id,
                           # Do NOT winsorize growth since restricted to [-2; 2]
                           # by construction
                           -matches("_gr1")), 
              .funs = funs(winsor(., 0.01))) %>%
    # Require at least T = 2 per gvkey-fqtr, i.e., 5 years of data with rho_X
    group_by(gvkey, fqtr) %>%
    mutate(n_obs = n()) %>%
    ungroup() %>%
    filter(n_obs >= 2) %>% 
    arrange(gvkey, datadate) %>%
    copy_to(pg, ., "aa_set", temporary = FALSE, 
            indexes = list(c("gvkey", "fyearq", "fqtr"), 
                           c("gvkey", "fqtr"), 
                           c("gvkey")))

comment <- 'CREATED USING azakolyukina/non_gaap/create_aa_set.R'
db_comment <- paste0("COMMENT ON TABLE est_set IS '",
                     comment, " ON ", Sys.time() , "'; ") 
dbGetQuery(pg, db_comment)

dbGetQuery(pg, "ALTER TABLE est_set OWNER TO non_gaap")
dbGetQuery(pg, "GRANT SELECT ON TABLE est_set TO non_gaap_access")
dbGetQuery(pg, "VACUUM ANALYZE aa_set")

dbDisconnect(pg)