# This code creates cik-gvkey link table for the Audit Analytics data.
# Bad links (many-to-one) verified manually using 10-K filings.

library(dplyr, warn.conflicts = FALSE)
library(lubridate)
library(RPostgreSQL)
library(googledrive)

rm(list = ls())
pg <- dbConnect(PostgreSQL())

schema_name <- "non_gaap"


# Download data ----

# Project schema
dbGetQuery(pg, paste("SET search_path TO ", schema_name, ", public"))
detail <- tbl(pg, "detail")

# Outside of project schema
wrds_cik <- tbl(pg, sql("SELECT * FROM ciq.wrds_cik"))
wrds_gvkey <- tbl(pg, sql("SELECT * FROM ciq.wrds_gvkey"))


# Merge CIK with GVKEY ----

# Determine first and last dates for CIKs
start <- wrds_cik %>%
  summarize(start = min(startdate, na.rm=TRUE)) %>%
  pull()

end <- wrds_cik %>%
  summarize(end = max(enddate, na.rm=TRUE)) %>%
  pull()

# CIK from WRDS
wrds_cik_alt <-
  wrds_cik %>%
  mutate(startdate = coalesce(startdate, start),
         enddate = coalesce(enddate, end)) %>%
  group_by(companyid, cik) %>%
  summarize(startdate = min(startdate, na.rm = TRUE),
            enddate = max(enddate, na.rm = TRUE)) %>%
  mutate(cik = as.integer(cik)) %>%
  compute()

# GVKEY from WRDS
wrds_gvkey_alt <-
  wrds_gvkey %>%
  mutate(startdate = coalesce(startdate, start),
         enddate = coalesce(enddate, end)) %>%
  group_by(companyid, gvkey) %>%
  summarize(startdate = min(startdate, na.rm = TRUE),
            enddate = max(enddate, na.rm = TRUE)) %>%
  compute()

# Merge WRDS CIK with Compustat CIK
wrds_cik_alt <- 
  wrds_cik_alt %>%
  select(companyid, cik, startdate, enddate) %>%
  distinct() %>%
  inner_join(wrds_gvkey_alt,
             by = "companyid",
             suffix = c("_cik", "_gvkey")) %>% 
  filter(enddate_gvkey > startdate_cik,
         enddate_cik   > startdate_gvkey) %>%
  mutate(first_date = greatest(startdate_cik, startdate_gvkey),
         last_date = least(enddate_cik, enddate_gvkey),
         first_date = if_else(first_date == start, NA, first_date),
         last_date  = if_else(last_date  == end,   NA, last_date)) %>%
  filter(last_date > first_date | is.na(last_date) | is.na(first_date)) %>%
  select(cik, gvkey, first_date, last_date) %>%
  compute()

# Keep only firms included in Audit Analytics data
detail_alt <- 
  detail %>% 
  select(cik = cik_code, date) %>%
  distinct() %>%
  inner_join(wrds_cik_alt, by = 'cik') %>%
  filter((date >= first_date) | is.na(first_date),
         (date <= last_date)  | is.na(last_date)) %>%
  select(-date) %>%
  distinct() %>%
  compute()

# Fill in blank start and end dates with sample start and end
detail_alt <- 
  detail_alt %>%
  ungroup() %>%
  mutate(first_date = coalesce(first_date, start),
         last_date = coalesce(last_date, end)) %>%
  compute()


# Clean up bad links ----

# List of bad matches - MUST BE UPDATED
bad_matches <-
    detail_alt %>% 
    group_by(gvkey) %>% 
    filter(n()>1) %>% 
    compute()

# Load file of corrected links
temp <- 
    tempfile(fileext = '.csv')
    drive_download(
      drive_get(id = "1rGJs8R5IuAwDYofdDRTi7saR9QdpenAt99uh_hUW_q4"),
      path = temp,
      overwrite = TRUE)
resolved <- read.csv(temp)

# Clean up google document
resolved <-
  resolved %>% 
  select(-comment) %>% 
  mutate(gvkey = as.character(gvkey),
         first_date = as.Date(first_date),
         last_date = as.Date(last_date)) %>% 
  as.data.frame()

# Create a temporary file in schema
resolved <- copy_to(pg, resolved, temporary=TRUE)


# Merge corrected links and save ----

dbGetQuery(pg, "DROP TABLE IF EXISTS detail_link")

detail_link <-
  dplyr::union(resolved, 
               detail_alt %>%
              anti_join(resolved, by="cik")) %>% 
  compute(name="detail_link", temporary=FALSE)


dbGetQuery(pg, paste("ALTER TABLE detail_link OWNER TO", schema_name))
dbGetQuery(pg, paste0("GRANT SELECT ON TABLE cik_gvkey TO ", schema_name, "_access"))

comment <- 'Created using create_detail_link.R'
sql <- paste0("COMMENT ON TABLE detail_link IS '",
              comment, " ON ", Sys.time() , "'")
rs <- dbGetQuery(pg, sql)

dbDisconnect(pg)
