# Link AA data to Compustat and convert per-share dollar values to unscaled values.

library(dplyr, warn.conflicts = FALSE)
library(lubridate)
library(RPostgreSQL)
library(psych)

rm(list = ls())

# Set up Postgre connection
pg <- dbConnect(PostgreSQL())

schema_name <- "non_gaap"

# Define PG data ----

# Project schema
rs <- dbGetQuery(pg, "SET search_path TO non_gaap, public")
dbGetQuery(pg, paste("SET search_path TO ", schema_name))
detail <- tbl(pg, "detail")
detail_link <- tbl(pg, "detail_link")

# Outside of project schema
fundq <- tbl(pg, sql("SELECT * FROM comp.fundq")) # Quarterly Compustat data

# Load GoogleDrive files ----

library(googlesheets4)

# Remove AA_taxonomies that cannot be classified
our_taxonomy <- read_sheet("1oc_qmyS8pRZuIxUepusdXc_nskus-iDyAlvbRDCHHAI") %>% filter(taxonomy != "NA")

# Metrics
metric_types_to_keep <- read_sheet("1U1DbOY0tn4cM2GArxSXl8nQVOdVn78F9NOHWsdhpWDo")

# Category type
category_type <- read_sheet("1lj7LY05WN1jNxnpIFr8Z-8K-vh5qb5wkpQcl7AyyVCc")

# Link AA data to Compustat ----

# Filter Compustat quarterly and keep the columns we will use
fundq_alt <- 
    fundq %>%
    select(gvkey, fqtr, fyearq, datadate, cshfdq, cshprq, cshoq, saleq,
           indfmt, datafmt, popsrc, consol, fic) %>% 
    filter(indfmt == 'INDL', datafmt == 'STD', popsrc == 'D', consol == 'C', fic == 'USA',
           !is.na(saleq), !is.na(gvkey), !is.na(fqtr), !is.na(fyearq)) %>% 
    select(-indfmt, -datafmt, -popsrc, -consol, -fic) %>%
    # Fill in missing number of shares
    mutate(n_shares = coalesce(cshfdq, cshprq, cshoq),
           fiscal_period = paste0('Q', fqtr, fyearq)) %>%
    # Drop observations with missing share data
    filter(!is.na(n_shares)) %>% 
    select(-cshfdq, -cshprq, -cshoq, -fqtr, -fyearq) %>% 
    compute()

# Add GVKEY to AA to Compustat 
detail_alt <- 
    detail %>% 
    select(cik_code, date, fiscal_period, metric_type, metric_key, 
           name_of_the_metric, aa_taxonomy, dollar_value, line_sequence_number, type_of_line) %>%
    # Keep only reconciliation items
    filter(type_of_line == 'Reconciliation') %>%
    select(-type_of_line) %>%
    # Add GVKEY to detail making sure CIK is valid between dates
    inner_join(select(detail_link, -companyid), 
               by = c('cik_code' = 'cik')) %>%
    filter(date >= first_date, date <= last_date) %>%
    # Drop dates for CIK linking
    select(-first_date, -last_date) %>% 
    compute()
  
# Clean up metric_type, unscale per-share values, clean up aa_taxonomies
detail_alt <- 
    detail_alt %>% 
    collect() %>% 
    # Remove unnecessary spaces and punctuation and capitalise first letters in metric_type
    mutate(metric_type = gsub('(\\b)([a-z])', '\\1\\U\\2', 
                              gsub('(^ | $)', '',
                                   gsub('\\s+', ' ', 
                                        gsub('[^a-zA-Z, \\/&-]', ' ', 
                                             metric_type))), 
                              perl = TRUE),
    # Replace names of some non-GAAP metrics that are synonyms
    metric_type = case_when(metric_type == 'Funds From Operations' ~ 'FFO',
                            metric_type == 'AFFO' ~ 'Adjusted FFO',
                            metric_type == 'AFFO Per Share' ~ 'Adjusted FFO Per Share', 
                            TRUE ~ metric_type),
    # Remove unnecessary spaces and punctuation and capitalise first letters in aa_taxonomy
    aa_taxonomy = gsub('(\\b)([a-z])', '\\1\\U\\2',
                       gsub('(^ | $)', '',
                            gsub('\\s+', ' ', 
                                 gsub('[^a-zA-Z, \\/&-]', ' ', aa_taxonomy))), 
                       perl = TRUE)) %>% 
    # Drop observations with missing metric_keys
    filter(!is.na(metric_key))
    
# Keep only earnings-related metric_type
detail_alt <- detail_alt %>%
    inner_join(metric_types_to_keep, copy=TRUE) %>%
    filter(keep == 1) %>%
    select(-keep)

# Clean up metrics
detail_alt <-
  detail_alt %>% 
  select(gvkey, cik_code, date, fiscal_period, metric_key, 
         metric_type, aa_taxonomy, dollar_value) %>% 
  # Sum adjustments by company, fiscal_period, and AA category and count lines
  group_by(gvkey, fiscal_period, metric_key, metric_type, aa_taxonomy) %>% 
  filter(date == max(date, na.rm = TRUE),
         dollar_value != 0) %>%
  summarise(combined_value = sum(dollar_value, na.rm = TRUE),
            n_lines = n(),
            cik_code = first(cik_code), 
              # ^ I checked and CIKs are always the same w/in group
            date = first(date)) %>% 
  ungroup()

# Determine condensed taxonomy and type of adjustment----

# Merge with detail_set_alt file
detail_alt <- 
    detail_alt %>% 
    collect() %>%
    # Merge aa_taxonomies to our taxonomies
    inner_join(our_taxonomy, by = "aa_taxonomy", copy = TRUE) %>%
    # Remove rows that sum to 0
    filter(combined_value != 0) %>% 
    # Contract rows to our taxonomies
    group_by(gvkey, fiscal_period, metric_key, metric_type, taxonomy) %>% 
    summarise(cik_code = first(cik_code),
              combined_value = sum(combined_value),
              n_lines = sum(n_lines)) %>% 
    ungroup()

# Merge with detail_set_alt file
detail_alt <- 
    detail_alt %>% 
    inner_join(category_type %>% select(-id), by = "taxonomy", copy = TRUE) %>% 
    compute()

# Join to Compustat ----
detail_set <- detail_alt %>% 
    # Use inner_join to only keep quarterly data
    inner_join(fundq_alt %>% collect(), by = c('fiscal_period', 'gvkey')) %>% 
    # Convert shares outstanding and sales from millions to dollars or shares
    mutate(n_shares = n_shares*1e6,
           saleq = saleq*1e6) %>% 
    # Remove rows that sum to 0
    filter(combined_value != 0) %>%
      # ^ This never happens when n_lines > 1 so we're good for print_stats_nongaap.Rnw
    # Unscale per-share values into dollars 
    mutate(value = case_when(grepl("(EPS|Per Share)", 
              metric_type) ~ as.double(combined_value*n_shares), TRUE ~ combined_value),
           value_share = value / saleq) %>%
    select(-combined_value) %>% 
    # Winsorize adjustments at 1-/99-
    mutate_at(.vars = vars(value, value_share), 
            .funs = funs(winsor(., 0.01)))

# Save data to pg
dbGetQuery(pg, "DROP TABLE IF EXISTS detail_set")

copy_to(pg, detail_set) %>%
  compute(name="detail_set", temporary=FALSE)

comment <- 'CREATED USING create_detail_set.R IN GitHub azakolyukina/non_gaap'
db_comment <- paste0("COMMENT ON TABLE detail_set IS '",
                     comment, " ON ", Sys.time() , "'; ")
rs <- dbExecute(pg, db_comment)

dbGetQuery(pg, "ALTER TABLE detail_set OWNER TO non_gaap")
dbGetQuery(pg, "GRANT SELECT ON TABLE detail_set TO non_gaap_access")
dbGetQuery(pg, "VACUUM ANALYZE detail_set")

dbDisconnect(pg)

