# This code computes CEO myopia following Edmans et al (2013) (https://www.nber.org/papers/w19407.pdf) 
# WARNING: Equilar data used here ends in 2016, i.e., the last fyear for mmratio is 2015

library(dplyr, warn.conflicts = FALSE)
library(RPostgreSQL)
library(tidyr)

pg <- dbConnect(PostgreSQL())
dbExecute(pg, "SET work_mem TO '10GB'") # Speeds-up creation of temporary tables

# Data ----

# Project schema
rs <- dbGetQuery(pg, "SET search_path TO non_gaap, public")

# Outside of project schema

delta_vega <- tbl(pg, sql("SELECT * FROM exec_comp_2016.delta_vega")) # Pre-computed equilar data on Executive Delta
company_fyear <- tbl(pg, sql("SELECT * FROM exec_comp_2016.company_fyear")) # Equilar-Compustat/CRSP link
executives <- tbl(pg, sql("SELECT * FROM exec_comp_2016.executives")) # Executives in Equilar
equity <- tbl(pg, sql("SELECT * FROM exec_comp_2016.equity")) # Executives in Equilar
equity_calc <- tbl(pg, sql("SELECT * FROM exec_comp_2016.equity_calc")) # total deltas
proxy_company <- tbl(pg, sql("SELECT * FROM executive_2016.proxy_company")) # Equilar data on company information
fundq <- tbl(pg, sql("SELECT * FROM comp.fundq"))

# Get equity delta ----

# Get quarter ends
fundq_alt <- 
    fundq %>%
    select(gvkey, datadate, fyear = fyearq, fqtr) %>%
    group_by(gvkey, fyear, fqtr) %>%
    filter(datadate == max(datadate)) %>%
    ungroup()

company_fqtr <- 
    company_fyear %>%
    left_join(fundq_alt) %>%
    compute()

# WARNING: delta is per option x prc/100 as in Core and Guay(2001)
# delta = if_else(!is.na(z), exp(-div_yield*term)*pnorm(z)*(prc/100), NA_real_),
delta_vega_alt <- 
    delta_vega %>%
    left_join(equity %>%
                  filter(!is.na(expiration_date)) %>%
                  select(mgmt_yearly_exec_id, outstanding_equity_id,
                         company_id, executive_id, management_id, 
                         fiscal_year, fy_end, fyear, 
                         equity_amount, t_mature, type, expiration_date)) %>%
    select(mgmt_yearly_exec_id, company_id, executive_id, management_id,
           fiscal_year, fy_end, fyear,
           equity_amount, exercise_price, t_mature, type, delta, expiration_date) %>%
    mutate(exercise_price_str = sql("round(exercise_price::numeric, 2)::text")) %>%
    compute()

executives_alt <-
    executives %>%
    select(company_id, management_id, executive_id, 
           fiscal_year_id, fyear, fiscal_year,
           equilar_pay_rank, is_ceo) %>% 
    filter(is_ceo) %>% 
    compute()

# Compute newly vested options ----

# Edmans et al. (2013) (see Appendix A, B): Newly Vesting Options is the delta of newly-vesting options 
# in t+1 times stock price at t. Delta is computed using the Black-Scholes for 
# each pair of exercise price and expiration date, using the stock price at the end of t.

# WARNING: delta in delt_vega is as in Core and Guay (2001) and thus x prc/100 already(!)

# WARNING: processed Equilar data in exec_comp uniquely identifies type of 
# option tranches = {opt_new, opt_unex_unex, opt_unex_ex} it does NOT
# double counts opt_new as opt_unex_unex
# (Edmans et al. (2013) remove double-counting through the formula they use)

# Time {t} options that can vest in {t + 1} 
opts_t <- 
    delta_vega_alt %>% 
    filter(type %in% c("opt_unex_unex", "opt_new")) %>%
    mutate(expir_year = sql("date_part('year', expiration_date)"), 
           expir_month = sql("date_part('month', expiration_date)")) %>%
    # Aggregate by tranche
    group_by(mgmt_yearly_exec_id, company_id, executive_id, management_id, fyear, 
             exercise_price, exercise_price_str, t_mature, expiration_date, 
             expir_year, expir_month, delta) %>%
    summarise(equity_amount = sum(equity_amount)) %>%
    ungroup() %>%
    # Drop year-specific executive identifiers
    select(-mgmt_yearly_exec_id, -management_id) %>%
    compute()

# Time {t + 1} options 
opts_t1 <- 
    delta_vega_alt %>% 
    filter(type == "opt_unex_unex") %>%
    mutate(expir_year = sql("date_part('year', expiration_date)"), 
           expir_month = sql("date_part('month', expiration_date)")) %>% 
    # Aggregate by tranche
    group_by(mgmt_yearly_exec_id, company_id, executive_id, management_id, fyear, 
             exercise_price, exercise_price_str, t_mature, expiration_date, 
             expir_year, expir_month) %>%
    summarise(equity_amount = sum(equity_amount)) %>%
    ungroup() %>%
    # Drop year-specific executive identifiers
    select(-mgmt_yearly_exec_id, -management_id) %>%
    compute()

# Compute change in vesting options from t to t+1
# new_vest_num: the number of options in a given tranche that vest between t and t+1
# new_vest_val: the value of options in a given tranch that vest
# unvestedadj_val: the value of unvested options at t, max((unvested - new_vest_num)*delta, 0) 
opts_vest <- 
    opts_t %>%
    # Try to match on {excercise_price, expir_year, expir_month}
    left_join(opts_t1 %>%
                  mutate(fyear = fyear - 1) %>%
                  select(-expiration_date, -exercise_price, - t_mature), 
              by = c("company_id", "executive_id", "fyear",  
                     "exercise_price_str", "expir_year", "expir_month"), 
              suffix = c("_t", "_t1")) %>%
    # Try to match on {excercise_price, t_mature}
    left_join(opts_t1 %>%
                  mutate(fyear = fyear - 1, 
                         t_mature = t_mature + 1) %>%
                  select(-expiration_date, -exercise_price, -expir_year, -expir_month), 
              by = c("company_id", "executive_id", "fyear",  
                     "exercise_price_str", "t_mature")) %>%
    # Replace missing matches on {excercise_price, expir_year, expir_month}
    # with matches on {excercise_price, t_mature}
    mutate(equity_amount_t1 = coalesce(equity_amount_t1, equity_amount)) %>%
    select(-equity_amount) %>%
    # If NOT {company_id, executive_id} last year, replace NA equity_amount_t1 with 0
    group_by(company_id, executive_id) %>%
    mutate(fyear_max = max(fyear)) %>%
    ungroup() %>%
    mutate(equity_amount_t1 = if_else(fyear != fyear_max, 
                                      coalesce(equity_amount_t1, 0), equity_amount_t1)) %>%
    # Newly-vesting options
    mutate(new_vest_num = equity_amount_t - equity_amount_t1) %>%
    mutate(new_vest_val = new_vest_num*delta, 
           unvestedadj_val = greatest((equity_amount_t - new_vest_num)*delta, 0)) %>%
    compute()

# Aggregate and compute vesting ratio at quarterly frequency
# WARNING: In opts_vest, fyear = t and shows options vested between t and t + 1,
# Vesting timeline: (1) fyear XXX -> XXX Q4, if no expiration date; 
# (2) assume vesting at anniversery of expiration date
company_fqtr_alt <- 
    company_fqtr %>%
    mutate(fyear_alt = if_else(fqtr == 4, fyear, fyear - 1), 
           fqtr_alt = 4) %>%
    left_join(company_fqtr %>%
                  filter(fqtr == 4) %>%
                  select(gvkey, fqtr_alt = fqtr, fyear_alt = fyear, datadate_alt = datadate)) %>%
    mutate(calyr_alt = sql("date_part('year', datadate_alt)")) %>%
    compute()
    
opts_vest_qtr <- 
    company_fqtr_alt %>%
    # Get the start of the next quarter
    group_by(gvkey) %>%
    arrange(fyear, fqtr) %>%
    mutate(datadate_lead = lead(datadate)) %>%
    ungroup() %>%
    left_join(opts_vest %>%
                  mutate(fqtr_alt = 4) %>%
                  rename(fyear_alt = fyear)) %>%
    # Get expiration anniversery AFTER datadate_alt
    mutate(expir_day = sql("date_part('day', expiration_date)"),
           expir_month = sql("date_part('month', expiration_date)")) %>%
    mutate(expir_ann0 = sql("to_Date((calyr_alt - 1)::varchar ||' '|| expir_month::varchar ||' '|| LEAST(expir_day, 28)::varchar, 'YYYY MM DD')"), 
           expir_ann1 = sql("to_Date(calyr_alt::varchar ||' '|| expir_month::varchar ||' '|| LEAST(expir_day, 28)::varchar, 'YYYY MM DD')"), 
           expir_ann2 = sql("to_Date((calyr_alt + 1)::varchar ||' '|| expir_month::varchar ||' '|| LEAST(expir_day, 28)::varchar, 'YYYY MM DD')")) %>%
    # Keep expir_ann immediately AFTER datadate_alt
    mutate(expir_ann = if_else(expir_ann0 > datadate_alt, expir_ann0, NA)) %>%
    mutate(expir_ann = if_else(is.na(expir_ann) & expir_ann1 > datadate_alt, expir_ann1, expir_ann)) %>%
    mutate(expir_ann = if_else(is.na(expir_ann) & expir_ann2 > datadate_alt, expir_ann2, expir_ann)) %>%
    # Assume vested at anniversery of expiration_date AFTER datadate_alt
    mutate(new_vest_valq = if_else(datadate < expir_ann & expir_ann <= datadate_lead, new_vest_val, 0)) %>%
    # If expiration_date is empty, keep at the END of year t, i.e., XXX Q4
    mutate(new_vest_valq = if_else(is.na(expir_ann) & fqtr == 4, new_vest_val, new_vest_valq)) %>%
    # Set missing new_vest_valq to 0, unless fyear_alt == fyear_max
    mutate(new_vest_valq = if_else(is.na(new_vest_valq) & fyear_alt != fyear_max, 0, new_vest_valq)) %>%
    mutate(new_vest_valq = if_else(is.na(new_vest_val), NA, new_vest_valq)) %>%
    distinct() %>%
    compute()

# Aggregate by quarter and compute mmratio
opts_mm <- 
    opts_vest_qtr %>%
    group_by(company_id, executive_id, gvkey, fyear, fqtr, datadate) %>%
    summarise(new_vest_valq_sum = sum(new_vest_valq), 
              unvestedadj_val_sum = sum(unvestedadj_val)) %>%
    ungroup() %>%
    mutate(mmratio = new_vest_valq_sum/NULLIF(new_vest_valq_sum + unvestedadj_val_sum, 0)) %>%
    mutate(mmratio = if_else(new_vest_valq_sum == 0, 0, mmratio)) %>%
    compute()

# Add delta_vested for opt_unex_ex and shares held ----

equity_calc_alt <- 
    equity_calc %>%
    select(company_id, executive_id, fyear, delta_vested) %>%
    compute()

# Compute stock holding by CEO ----

equity_alt <-  
    equity %>%
    filter(type != "opt_new") %>%
    select(company_id, management_id, executive_id, mgmt_yearly_exec_id, fyear, fy = fy_end,
           type, equity_amount) %>%
    # Aggregate outstanding equity
    group_by(company_id, management_id, executive_id, mgmt_yearly_exec_id, fyear, fy, type) %>%
    summarize(equity_amount = sum(equity_amount)) %>%
    ungroup() %>%
    collect() %>%
    # Convert long to wide
    spread(type, equity_amount) %>%
    rename(own_shares = shares_held) %>%
    mutate(opt_unex = opt_unex_ex + opt_unex_unex) %>%
    copy_to(pg, ., name = "equity_alt1", temporary = TRUE)

# fiscal_year_id disambiguates between multiple obs. per fiscal_year
proxy_company_alt <- 
    proxy_company %>%
    select(fiscal_year_id, company_id, fiscal_year, shares_out_fy) %>%
    distinct() %>%
    compute(index = c("company_id", "fiscal_year", "fiscal_year_id"))

# Compute ownership fractions and CEO indicator
own_alt <- 
    executives_alt %>%
    left_join(equity_alt) %>%
    left_join(proxy_company_alt) %>%
    mutate(own_shares_fr = 1.0*own_shares/NULLIF(shares_out_fy, 0), 
           opt_unex_ex_fr = 1.0*opt_unex_ex/NULLIF(shares_out_fy, 0),  
           opt_unex_unex_fr = 1.0*opt_unex_unex/NULLIF(shares_out_fy, 0),  
           opt_unex_fr = 1.0*(opt_unex_ex + opt_unex_unex)/NULLIF(shares_out_fy, 0)) %>%
    # Extrapolate one period forward within executive_id
    mutate(own_shares_fr_lag = sql("lag(own_shares_fr) OVER (PARTITION BY company_id, executive_id ORDER BY fy)")) %>%
    mutate(own_shares_fr = coalesce(own_shares_fr, own_shares_fr_lag)) %>%
    select(-own_shares_fr_lag) %>%
    # For multiple {company_id, executive_id, fyear}, keep max fy
    group_by(company_id, executive_id, fyear) %>%
    filter(fy == max(fy)) %>%
    ungroup() %>%
    # For multiple {company_id, fyear}, keep min equilar_pay_rank
    group_by(company_id, fyear) %>%
    filter(equilar_pay_rank == min(equilar_pay_rank)) %>%
    ungroup() %>%
    # For multiple {company_id, executive_id, fyear}, keep min mgmt_yearly_exec_id. 
    # Eliminates duplicates that appear to be errors in Equilar
    group_by(company_id, executive_id, fyear) %>%
    filter(mgmt_yearly_exec_id == min(mgmt_yearly_exec_id)) %>%
    ungroup() %>%
    distinct() %>%
    compute()

# Merge data and merge down

dbGetQuery(pg, "DROP TABLE IF EXISTS executives")

# Timing for delta and ownership fraction is similar to mmratio, 
# i.e., FYEAR Q4, FYEAR + 1 Q1, FYEAR + 1 Q2, FYEAR + 1 Q3
# (at the end of t, the ownership for the period t to {t + 1})
exec_merged <- 
    company_fqtr_alt %>%
    left_join(own_alt %>%
                  mutate(fqtr_alt = 4) %>%
                  rename(fyear_alt = fyear)) %>%
    left_join(equity_calc_alt %>%
                  mutate(fqtr_alt = 4) %>%
                  rename(fyear_alt = fyear)) %>%
    left_join(opts_mm) %>%
    # If NOT {company_id, executive_id} last year, replace NA mmratio with 0
    # e.g., filter(company_id == 2 & executive_id == 452640 & fyear >= 2011)
    # only opt_unex-ex---NO unvested options
    group_by(company_id, executive_id) %>%
    mutate(fyear_max = max(fyear)) %>%
    ungroup() %>%
    # WARNING: set mmratio = 0 if not the last year for {company_id, executive_id}
    mutate(mmratio = if_else(mmratio < 0, NA, mmratio)) %>%
    mutate(mmratio = if_else(fyear != fyear_max, coalesce(mmratio, 0), mmratio)) %>%
    # WARNING: Equilar's fiscal_year is CALENDAR year
    # drop fiscal_year to avoid confusion
    select(company_id, executive_id, gvkey, fyear, fqtr, datadate,
           own_shares_fr, opt_unex_ex_fr, opt_unex_unex_fr, opt_unex_fr,
           mmratio, delta_vested,
           is_ceo, equilar_pay_rank) %>%
    # Require non-missing keys
    filter(!is.na(company_id) & !is.na(executive_id) & !is.na(gvkey)
           & !is.na(fyear) & !is.na(fqtr) & !is.na(datadate)) %>%
    distinct() %>%
    arrange(company_id, executive_id, datadate) %>%
    compute(name = "executives", temporary=FALSE)

db_comment <- paste0("CREATED USING create_executives from ",
                     "GitHub azakolyukina/non_gaap ON ", Sys.time())
dbGetQuery(pg, paste0("COMMENT ON TABLE executives IS '", db_comment, "'"))

dbGetQuery(pg, "ALTER TABLE executives OWNER TO non_gaap")
dbGetQuery(pg, "GRANT SELECT ON executives TO non_gaap_access")
dbGetQuery(pg, "VACUUM ANALYZE executives")

dbDisconnect(pg)
