# This code computes change in firm value following Bradshaw and Sloan (2002)
# For t-1,  prior quarter + 2 days; for t, current quarter + 1 day

#.rs.restartR()
#(.packages())
# rm(list = ls())

library(RPostgreSQL)
library(dplyr)

pg <- dbConnect(PostgreSQL())

# Define PG data ----

# Project schema
rs <- dbGetQuery(pg, "SET search_path TO non_gaap")
bcgw <- tbl(pg, "bcgw")

# Outside of project schema
fundq <- tbl(pg, sql("SELECT * FROM comp.fundq"))
dsf <- tbl(pg, sql("SELECT * FROM crsp.dsf"))
ccmxpf_linktable <- tbl(pg, sql("SELECT * FROM crsp.ccmxpf_linktable"))

# Set params ----

start_date <- "1998-01-01"
end_date <- "2100-01-01"
late_date <- '2100-01-01'

# Get the universe of {gvkey, datadate} for BCGW gvkeys ---
## RDQ	DATE	RDQ -- Report Date of Quarterly Earnings

fundq_alt <- 
    fundq %>%
    select(gvkey, datadate, rdq) %>%
    filter(start_date <= datadate & datadate <= end_date) %>%
    # Replace missing rdq with datadate + median diff between datadate and rdq
    mutate(rdq = if_else(is.na(rdq), sql("datadate + integer '40'"), rdq)) %>%
    # Compute rdq + 1 and rdq + 2 days
    mutate(rdq_plus1 = sql("rdq + integer '1'"), 
           rdq_plus2 = sql("rdq + integer '2'")) %>%
    distinct() %>%
    compute()

bcgw_alt <- 
    bcgw %>%
    select(gvkey) %>%
    distinct() %>%
    left_join(fundq_alt) %>%
    # Keep the latest RDQ date
    group_by(gvkey, datadate) %>%
    filter(rdq == max(rdq)) %>%
    ungroup() %>%
    compute()

# Daily price data and split adjustment factors ----

# CRSP prices and split-adjustment factors
# WARNING: use CFACPR (Cumulative Factor to Adjust Price) to account for stock splits and stock dividends
# Adjusted Price = PRC / CFACPR
# Adjusted Shares = SHROUT * CFACSHR
# PRC is the closing price or the negative(!) bid/ask average for a trading day
# SHROUT is the number of publicly held shares, recorded in thousands(!)
dsf_alt <- 
    dsf %>%
    filter(start_date <= date & date <= end_date) %>%
    select(permno, date, prc, cfacpr, shrout, cfacshr) %>%
    mutate(prc = abs(prc)) %>%
    mutate(mcap = abs(prc * shrout/(10^3))) %>% # Convert to millions
    # Get permno-gvkey link 
    left_join(ccmxpf_linktable %>%
                  mutate(linkenddt = coalesce(linkenddt, late_date)) %>%
                  filter(linkprim %in% c("P", "C"), usedflag == 1L) %>%
                  rename(permno = lpermno)) %>%
    # Keep valid links only
    filter(linkdt < date & date <= linkenddt) %>%
    select(permno, gvkey, date, mcap, linkdt, linkenddt) %>%
    compute()

# 5-day interval for merge to match future dates
dsf_alt_long <- 
    dsf_alt %>%
    mutate(date_alt = date) %>%
    union(., dsf_alt %>% mutate(date_alt = sql("date - interval '1 day'"))) %>%
    union(., dsf_alt %>% mutate(date_alt = sql("date - interval '2 day'"))) %>%
    union(., dsf_alt %>% mutate(date_alt = sql("date - interval '3 day'"))) %>%
    union(., dsf_alt %>% mutate(date_alt = sql("date - interval '4 day'"))) %>%
    union(., dsf_alt %>% mutate(date_alt = sql("date - interval '5 day'"))) %>%
    compute()

# Merge firm value to earnings reporting dates ----

value_plus1 <- 
    bcgw_alt %>%
    left_join(dsf_alt_long %>%
                  rename(rdq_plus1 = date_alt)) %>%
    filter(rdq_plus1 <= date) %>%
    group_by(permno, rdq_plus1) %>%
    filter(date == min(date)) %>%
    ungroup() %>%
    # For {gvkey, datadate} duplicates, keep rdq in the link period
    filter(linkdt < rdq_plus1 & rdq_plus1 <= linkenddt) %>%
    compute()

value_plus2 <- 
    bcgw_alt %>%
    left_join(dsf_alt_long %>%
                  rename(rdq_plus2 = date_alt)) %>%
    filter(rdq_plus2 <= date) %>%
    group_by(permno, rdq_plus2) %>%
    filter(date == min(date)) %>%
    ungroup() %>%
    # For {gvkey, datadate} duplicates, keep rdq in the link period
    filter(linkdt < rdq_plus2 & rdq_plus2 <= linkenddt) %>%
    compute()

# Compute lag for _plus2
value_plus2_alt <- 
    value_plus2 %>%
    group_by(gvkey) %>%
    arrange(datadate) %>%
    mutate(mcap_lag = lag(mcap), 
           rdq_plus2_lag = lag(rdq_plus2), 
           date_lag = lag(date)) %>%
    # Set to NA if too close or too far
    mutate(mcap_lag = if_else(30 <= (rdq - rdq_plus2_lag) & (rdq - rdq_plus2_lag) <= 150, 
                              mcap_lag, NA)) %>%
    ungroup() %>%
    compute()

# Write to PG ----

dbGetQuery(pg, "DROP TABLE IF EXISTS value")

value <-
    value_plus1 %>%
    select(gvkey, permno, datadate, rdq, date, mcap) %>%
    left_join(value_plus2_alt %>%
                  select(gvkey, datadate, date_lag, mcap_lag)) %>%
    # For {gvkey, datadate} duplicates, keep the latest date_lag
    group_by(gvkey, datadate) %>%
    filter(date_lag == max(date_lag)) %>%
    ungroup() %>%
    arrange(gvkey, datadate) %>%
    compute(name = "value", temporary = FALSE,
            indexes=list(c("gvkey", "datadate"), c("gvkey")))

dbGetQuery(pg, "ALTER TABLE value OWNER TO non_gaap")
dbGetQuery(pg, "GRANT SELECT ON TABLE value TO non_gaap_access")
dbGetQuery(pg, "VACUUM ANALYZE value")

comment <- 'CREATED USING create_value.R  from GitHub azakolyukina/non_gaap'
db_comment <- paste0("COMMENT ON TABLE firms IS '",
                     comment, " ON ", Sys.time() , "'; ") 
dbGetQuery(pg, db_comment)

dbDisconnect(pg)
