function sim_model(obs, t, qseed, exog_uniform,
                   maxq, minq, q_num, distq, qgrid,
                   wmax, bmax, bmin, delta, tburn,
                   exog_num, exogval, tran_mat, w_star, b_star, vf, bunchtol)
    # This funciton simulates data for a given set of parameter values

    # Draw random numbers
    qseed = qseed.*(maxq - minq) .+ minq # set values of q for the initial period

    # Compute CDF of transition matrices
    cdf_mat       = zeros(exog_num, exog_num)
    cdf_mat[:, 1] = tran_mat[:, 1] # set first column to transition matrices

    for jj in 1:exog_num
        for ii in 2:exog_num - 1
            @inbounds cdf_mat[jj, ii] = cdf_mat[jj, ii - 1] + tran_mat[jj,ii]
        end
        cdf_mat[jj, end] = 1.0  # set last column to 1.0 to ensure no draws are above
    end

    # Initialize simulated data
    exogpos       = Array{Int64}(undef, obs, t)
    exogpos[:,1] .= floor(exog_num/2) # initialize exogenous process close to middle

    #Loop through each observation to compute the exogenous parameter position
    for jj in 1:obs
        for ii in 2:t
            prior_pos = exogpos[jj, ii - 1]
            ll = 1
            while exog_uniform[jj, ii] > cdf_mat[prior_pos, ll]
                ll += 1
            end
            exogpos[jj, ii] = ll
        end
    end

    # Initialize the matrices of actions and endogenous state (i.e., capital)
    bsim     = Array{Float64}(undef, obs, t) # bias
    wsim     = Array{Float64}(undef, obs, t) # investment
    vfsim    = Array{Float64}(undef, obs, t) # value
    qsim     = Array{Float64}(undef, obs, t) # capital
    nu_ysim  = Array{Float64}(undef, obs, t) # productivity shock
    nu_pisim = Array{Float64}(undef, obs, t) # accounting shock

    # Set first period q using seeded values
    qsim[:, 1] = qseed

    # Loop through all firms and periods to determine states and optimal actions
    for jj in 1:obs
        for ii in 1:t

            # Location of observation in state space
            indq, wtq = weightss(qgrid, qsim[jj, ii], minq, maxq, q_num, distq)

            # Optimal choices
            wsim[jj, ii]  = interp(w_star, b_star, indq, wtq, exogpos[jj,ii])
            bsim[jj, ii]  = interp(b_star, b_star, indq, wtq, exogpos[jj,ii])
            vfsim[jj, ii] = interp(vf, b_star, indq, wtq, exogpos[jj,ii])

            # Matrix of state variables
            if ii < t
                qsim[jj, ii + 1] = (1.0 - delta)*qsim[jj, ii] + wsim[jj, ii] # q′ for grid
            end
            nu_ysim[jj, ii]  = exogval[exogpos[jj, ii], 1] # productivity shock
            nu_pisim[jj, ii] = exogval[exogpos[jj, ii], 2] # accounting shock

        end
    end

    # Create bunching flags
    bunch_flag = Array{Int}(undef, 5, 1) # number of bunch_flag conditions to check
    bunch_flag = repeat([0], 5)

    # Isolate observations used in moment calculations to compute bunching
    qend = qsim[:,tburn+1:end-1] # capital
    wend = wsim[:,tburn+1:end-1] # investment
    bend = bsim[:,tburn+1:end-1] # bias

    # Determine whether the fraction at end points high
    if length(qend[qend .> (maxq - 0.0001)])/(obs*(t - tburn - 1)) > bunchtol # bunching on capital upper bound
        bunch_flag[1] = 1
    end

    if length(qend[qend .< (minq + 0.0001)])/(obs*(t - tburn - 1)) > bunchtol # bunching on capital lower bound
        bunch_flag[2] = 1
    end

    if length(wend[wend .== wmax])/(obs*(t - tburn - 1)) > bunchtol  # bunching on investment upper bound
        bunch_flag[3] = 1
    end

    if length(bend[bend .== bmax])/(obs*(t - tburn - 1)) > bunchtol  # bunching on bias upper bound
        bunch_flag[4] = 1
    end

    # WARNING: swtich off when bmin == 0
    if (bmin != 0.0 && length(bend[bend .== bmin])/(obs*(t - tburn - 1)) > bunchtol)  # bunching on bias lower bound
        bunch_flag[5] = 1
    elseif (bmin == 0.0)
        bunch_flag[5] = 0
    end

    # qsim - capital obs x t
    # nu_ysim - productivity shock  obs x t
    # nu_pisim - accounting noise  obs x t
    # wsim - investment  obs x t
    # bsim - bias  obs x t
    # bunch_flag - binary variable to indicate whether there is bunching
    return qsim, nu_ysim, nu_pisim, wsim, bsim, vfsim, bunch_flag

end
